@@ -277,7 +277,7 @@ add_old_servers (gpointer key,
 
 
 static gboolean
-server_cache_reap_cb ()
+server_cache_reap_cb (void)
 {
      	int size;
 	GPtrArray *servers;
@@ -309,7 +309,7 @@ schedule_server_cache_reap (void)
 {
 	if (server_cache_reap_timeout == 0) {
 		server_cache_reap_timeout = g_timeout_add (SERVER_CACHE_REAP_TIMEOUT*1000,
-							   server_cache_reap_cb, NULL);
+							   (GSourceFunc)server_cache_reap_cb, NULL);
 	}
 }
 
@@ -838,7 +838,7 @@ init_auth (GnomeVFSURI *uri)
 }
 
 static gboolean
-auth_failed ()
+auth_failed (void)
 {
 	return cache_access_failed &&
 		(errno == EACCES || errno == EPERM) &&
@@ -884,7 +884,7 @@ auth_fn (const char *server_name, const char *share_name,
 		tmp = strchr (current_toplevel_uri->user_name, ';');
 		if (tmp != NULL) {
 			domain = g_strndup (current_toplevel_uri->user_name,
-					    current_toplevel_uri->user_name - tmp);
+					    tmp - current_toplevel_uri->user_name);
 			username = g_strdup (tmp + 1);
 		} else {
 			username = g_strdup (current_toplevel_uri->user_name);
@@ -1709,7 +1709,8 @@ do_read_directory (GnomeVFSMethod *method,
 	switch (entry->smbc_type) {
 	case SMBC_FILE_SHARE:
 		file_info->valid_fields = file_info->valid_fields
-			| GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE;
+			| GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE
+			| GNOME_VFS_FILE_INFO_FIELDS_TYPE;
 		file_info->type = GNOME_VFS_FILE_TYPE_DIRECTORY;
 		file_info->mime_type = g_strdup ("x-directory/smb-share");
 		break;
